#!/usr/bin/perl -w -I.
use strict;

package ODL::DBReview::DBBuilder;


# use lib '..';
use Pure::EZXML;
use DBI;


# constructor and explicit destructor

sub new
{
   my ($classname, $database, $dbusername, $dbpassword, $table) = @_;

   my $self = {
      threadepoch => 20,
      database    => $database,
      dbusername  => $dbusername,
      dbpassword  => $dbpassword,
      table       => $table,
   };

   bless $self, $classname;

   $self->{'dbh'} = $self->openDatabase ($self->{'database'}, $self->{'dbusername'}, 
                                         $self->{'dbpassword'}, $self->{'table'});

   return $self;
}


# open database and create if necessary
sub openDatabase
{
   my ($self, $database, $dbusername, $dbpassword, $table) = @_;
   
   my $dbh = DBI->connect ($database, $dbusername, $dbpassword); # , { PrintError => 0 });

   # check for existance of index
   my $sth = $dbh->prepare ("select internal_id from $table"."sr limit 1");
   $sth->execute;
   my @row = $sth->fetchrow_array;
   $sth->finish;
   if ($#row > -1)
   {
      return $dbh;
   }

   # make new tables in database
   $sth = $dbh->do ("create table $table"."sr (".
                    "type char(1) not null, ".
                    "title varchar(255) not null, ".
                    "internal_id varchar(255) not null, ".
                    "status varchar(20), ".
                    "submitter varchar(255), ".
                    "baseURL varchar(255), ".
                    "id varchar(255), ".
                    "metadataPrefix varchar(255), ".
                    "parent varchar(255), ".
                    "level integer not null, ".
                    "thread integer not null ".
                    ")");

   $sth = $dbh->do ("create index internal_id on $table"."sr (internal_id)");
   $sth = $dbh->do ("create index parent on $table"."sr (parent)");
   $sth = $dbh->do ("create index thread on $table"."sr (thread)");

   $sth = $dbh->do ("create table $table"."editor (".
                    "name varchar(255) not null, ".
                    "role varchar(255) not null, ".
                    "id varchar(255) not null, ".
                    "internal_id varchar(255) not null, ".
                    "deadline varchar(255) not null, ".
                    "parent varchar(255) not null, ".
                    "thread integer not null ".
                    ")");

   $sth = $dbh->do ("create index parent on $table"."editor (parent)");
   $sth = $dbh->do ("create index thread on $table"."editor (thread)");

   $sth = $dbh->do ("create table $table"."review (".
                    "baseURL varchar(255) not null, ".
                    "id varchar(255) not null, ".
                    "metadataPrefix varchar(255) not null, ".
                    "internal_id varchar(255) not null, ".
                    "editor varchar(255) not null, ".
                    "parent varchar(255) not null, ".
                    "thread integer not null ".
                    ")");

   $sth = $dbh->do ("create index parent on $table"."review (parent)");
   $sth = $dbh->do ("create index thread on $table"."review (thread)");
                    
   $sth = $dbh->do ("create table $table"."draft (".
                    "internal_id varchar(255) not null, ".
                    "title varchar(255) not null, ".
                    "baseURL varchar(255) not null, ".
                    "id varchar(255) not null, ".
                    "metadataPrefix varchar(255) not null, ".
                    "parent varchar(255) not null, ".
                    "thread integer not null ".
                    ")");
                    
   $sth = $dbh->do ("create index parent on $table"."draft (parent)");
   $sth = $dbh->do ("create index thread on $table"."draft (thread)");

   return $dbh;
}


sub dispose
{
   my ($self) = @_;

   $self->{'dbh'}->disconnect;
}


sub zap
{
   my ($self) = @_;

   $self->{'dbh'}->do ("drop table $self->{'table'}sr");
   $self->{'dbh'}->do ("drop table $self->{'table'}editor");
   $self->{'dbh'}->do ("drop table $self->{'table'}draft");
   $self->{'dbh'}->do ("drop table $self->{'table'}review");
   
   $self->{'dbh'}->disconnect;
   $self->{'dbh'} = $self->openDatabase ('DBI:mysql:dbreview', 'hussein', '', $self->{'table'});
}


sub isEditor
{
   my ($self, $userid, $resourceid) = @_;
   
   # counters to check for bootstrap condition
   my ($numeditors, $numparents) = (0, 0);

   # iterate over editor nodes for the resource node
   my $sth = $self->{'dbh'}->prepare ("select id,role from $self->{'table'}editor where parent=\'$resourceid\'");
   $sth->execute;
   while ( my $rowref = $sth->fetchrow_hashref )
   {
      if (($rowref->{'id'} eq $userid) && 
          ($rowref->{'role'} ne 'Reviewer'))
      {
         $sth->finish;
         return 1;
      }
      $numeditors++;
   }
   $sth->finish;
   
   # get parent node and recurse
   $sth = $self->{'dbh'}->prepare ("select parent from $self->{'table'}sr where internal_id=\'$resourceid\'");
   $sth->execute;
   while ( my $rowref = $sth->fetchrow_hashref )
   {
      if ($rowref->{'parent'} ne '')
      {
         $sth->finish;
         return $self->isEditor ($userid, $rowref->{'parent'});
      }
      $numparents++;
   }
   $sth->finish;
   
   # return bootstrap result for very first editor for root node
   if (($numeditors == 0) && ($numparents == 1))
   { 
      return 1;
   }

   return 0;
}
   

sub getsrThreadNumber
{
   my ($self, $parent) = @_;
   
   my $threadnumber = 0;
   my $level = 1;
   
   # get default parent parameter
   if ((! defined $parent) || ($parent eq ''))
   {
      $parent = '';
   }

   # find parent thread number
   my $sth = $self->{'dbh'}->prepare ("select thread,level from $self->{'table'}sr where internal_id=\'$parent\'");
   $sth->execute;
   my $rowref = $sth->fetchrow_arrayref;
   $sth->finish;

   if (($rowref) && ($#$rowref == 1) && ($rowref->[0] ne ''))
   # found a parent thread number
   {
      my $parentthreadnumber = $rowref->[0];
      $level = $rowref->[1] + 1;
      # find next entry in list
      $sth = $self->{'dbh'}->prepare ("select max(thread) from $self->{'table'}sr where thread<$parentthreadnumber");
      $sth->execute;
      my $nextthreadnumber = ($sth->fetchrow_array)[0];
      $sth->finish;
      if ($nextthreadnumber eq '')
      {
         $nextthreadnumber = 0;
      }
      
      # do a shift if necessary
      if (($nextthreadnumber + 1) >= $parentthreadnumber)
      {
         $self->{'dbh'}->do ("update $self->{'table'}sr set thread=thread+$self->{'threadepoch'} where thread>=$parentthreadnumber");
         $parentthreadnumber += $self->{'threadepoch'};
      }

      $threadnumber = int (($parentthreadnumber + $nextthreadnumber) / 2);
   }
   else
   # no parent thread number, so create new thread with large value
   {
      # search for current maximum thread number
      $sth = $self->{'dbh'}->prepare ("select max(thread) from $self->{'table'}sr");
      $sth->execute;
      my $maxthread = ($sth->fetchrow_array)[0];
      $sth->finish;

      if ($maxthread eq '')
      {
         $maxthread = 0;
      }
      $threadnumber = $maxthread + $self->{'threadepoch'};
   }
   
   ($threadnumber, $level);
}


sub add
{
   my ($self, $fragment, $identifier) = @_;

   my $parser = new Pure::EZXML;
   my $document = $parser->parse ($fragment);

   foreach my $odl_review ($document->getElementsByTagName ('odl_review', 0))
   {
      my $actor = $odl_review->getElementsByTagName ('actor', 0)->item(0)
                             ->getChildNodes->toString;
      foreach my $editor ($odl_review->getElementsByTagName ('editor', 0))
      {
         foreach my $assign ($editor->getElementsByTagName ('assign', 0))
         {
            my $role = $assign->getElementsByTagName ('role', 0)->item(0)
                                  ->getChildNodes->toString;
            my $name = $assign->getElementsByTagName ('name', 0)->item(0)
                                  ->getChildNodes->toString;
            my $id = $assign->getElementsByTagName ('id', 0)->item(0)
                                  ->getChildNodes->toString;
            my $resource = $assign->getElementsByTagName ('resource_id', 0)->item(0)
                                  ->getChildNodes->toString;
            my $deadline = $assign->getElementsByTagName ('deadline', 0)->item(0)
                                  ->getChildNodes->toString;
            if ($self->isEditor ($actor, $resource))
            {
               $self->{'dbh'}->do ("insert into $self->{'table'}editor set ".
                                   "name=\'$name\', ".
                                   "role=\'$role\', ".
                                   "id=\'$id\', ".
                                   "internal_id=\'$identifier\', ".
                                   "deadline=\'$deadline\', ".
                                   "parent=\'$resource\'"
                                  );
               return $resource;
            }
         }
         foreach my $decline ($editor->getElementsByTagName ('decline', 0))
         {
            my $id = $decline->getElementsByTagName ('id', 0)->item(0)->getChildNodes->toString;
            $self->{'dbh'}->do ("delete from $self->{'table'}editor where ".
                                "internal_id=\'$id\' and ".
                                "id=\'$actor\'"
                               );
            return '';
         }
         foreach my $revoke ($editor->getElementsByTagName ('revokerr', 0))
         {
            my $id = $revoke->getElementsByTagName ('id', 0)->item(0)->getChildNodes->toString;
            if ($self->isEditor ($actor, $id))
            {
               $self->{'dbh'}->do ("delete from $self->{'table'}editor where ".
                                   "internal_id=\'$id\'"
                                  );
               return '';
            }
         }
      }
      foreach my $resource ($odl_review->getElementsByTagName ('resource', 0))
      {
         foreach my $submit ($resource->getElementsByTagName ('submit', 0))
         {
            my $title = $submit->getElementsByTagName ('title', 0)->item(0)
                                  ->getChildNodes->toString;
            my $baseURL = $submit->getElementsByTagName ('baseURL', 0)->item(0)
                                  ->getChildNodes->toString;
            my $id = $submit->getElementsByTagName ('id', 0)->item(0)
                                  ->getChildNodes->toString;
            my $metadataPrefix = $submit->getElementsByTagName ('metadataPrefix', 0)->item(0)
                                  ->getChildNodes->toString;
            my $section = $submit->getElementsByTagName ('section_id', 0)->item(0)
                                  ->getChildNodes->toString;
            
            # add resource to an existing and open section
            my $sth = $self->{'dbh'}->prepare ("select status from $self->{'table'}sr where internal_id=\'$section\' and type=\'s\'");
            $sth->execute;
            my $rowref = $sth->fetchrow_arrayref;
            $sth->finish;
            if (($rowref) && ($#$rowref == 0) && ($rowref->[0] eq 'open'))
            { 
               my ($threadnumber, $level) = $self->getsrThreadNumber ($section);
               $self->{'dbh'}->do ("insert into $self->{'table'}sr set ".
                                   "title=\'$title\', ".
                                   "internal_id=\'$identifier\', ".
                                   "status=\'\', ".
                                   "type=\'r\', ".
                                   "parent=\'$section\', ".
                                   "baseURL=\'$baseURL\', ".
                                   "id=\'$id\', ".
                                   "metadataPrefix=\'$metadataPrefix\', ".
                                   "submitter=\'$actor\', ".
                                   "level=$level, ".
                                   "thread=$threadnumber"
                                  );
               return $section;
            }

            # add the resource as a newer version and convert the old version to a draft
            $sth = $self->{'dbh'}->prepare ("select status,submitter,title,baseURL,id,metadataPrefix from $self->{'table'}sr where internal_id=\'$section\' and type=\'r\'");
            $sth->execute;
            $rowref = $sth->fetchrow_hashref;
            $sth->finish;
            if (($rowref) && (defined $rowref->{'status'}) && ($rowref->{'status'} eq 'resubmit') && 
                (defined $rowref->{'submitter'}) && ($rowref->{'submitter'} eq $actor))
            {
               $self->{'dbh'}->do ("insert into $self->{'table'}draft set ".
                                   "internal_id=\'$identifier\', ".
                                   "title=\'$rowref->{'title'}\', ".
                                   "baseURL=\'$rowref->{'baseURL'}\', ".
                                   "id=\'$rowref->{'id'}\', ".
                                   "metadataPrefix=\'$rowref->{'metadataPrefix'}\', ".
                                   "parent=\'$section\'"
                                  );
               $self->{'dbh'}->do ("update $self->{'table'}sr set ".
                                   "title=\'$title\', ".
                                   "baseURL=\'$baseURL\', ".
                                   "id=\'$id\', ".
                                   "metadataPrefix=\'$metadataPrefix\', ".
                                   "status=\'\' where ".
                                   "internal_id=\'$section\'"
                                  );
               return $section;
            }
         }
         foreach my $move ($resource->getElementsByTagName ('move', 0))
         {
            my $id = $move->getElementsByTagName ('id', 0)->item(0)
                                  ->getChildNodes->toString;
            my $section = $move->getElementsByTagName ('section_id', 0)->item(0)
                                  ->getChildNodes->toString;
            if ($self->isEditor ($actor, $id))
            {
               my ($threadnumber, $level) = $self->getsrThreadNumber ($section);
               $self->{'dbh'}->do ("update $self->{'table'}sr set ".
                                   "thread=$threadnumber, ".
                                   "parent=\'$section\', ".
                                   "level=$level where ".
                                   "internal_id=\'$id\'"
                                  );
               return $id;
            }
         }
         foreach my $addsection ($resource->getElementsByTagName ('addsection', 0))
         {
            my $name = $addsection->getElementsByTagName ('name', 0)->item(0)
                                  ->getChildNodes->toString;
            my $sections = '';
            if ($addsection->getElementsByTagName ('section_id', 0)->getLength > 0)
            {
               $sections = $addsection->getElementsByTagName ('section_id', 0)->item(0)
                                  ->getChildNodes->toString;
            }
            my ($threadnumber, $level) = $self->getsrThreadNumber ($sections);
            $self->{'dbh'}->do ("insert into $self->{'table'}sr set ".
                                "title=\'$name\', ".
                                "internal_id=\'$identifier\', ".
                                "status=\'\', ".
                                "type=\'s\', ".
                                "parent=\'$sections\', ".
                                "level=$level, ".
                                "thread=$threadnumber"
                               );
            return $sections;
         }
         foreach my $status ($resource->getElementsByTagName ('changestatus', 0))
         {
            my $id = $status->getElementsByTagName ('id', 0)->item(0)
                                  ->getChildNodes->toString;
            my $newstate = $status->getElementsByTagName ('status', 0)->item(0)
                                  ->getChildNodes->toString;
            if ($self->isEditor ($actor, $id))
            {
               if ($newstate eq 'underreview')
               {
                  $newstate = '';
               }
               $self->{'dbh'}->do ("update $self->{'table'}sr set ".
                                   "status=\'$newstate\' where ".
                                   "internal_id=\'$id\'"
                                  );
               return $id;
            }
         }
         foreach my $review ($resource->getElementsByTagName ('review', 0))
         {
            my $baseURL = $review->getElementsByTagName ('baseURL', 0)->item(0)
                                  ->getChildNodes->toString;
            my $id = $review->getElementsByTagName ('id', 0)->item(0)
                                  ->getChildNodes->toString;
            my $metadataPrefix = $review->getElementsByTagName ('metadataPrefix', 0)->item(0)
                                  ->getChildNodes->toString;
            my $section = $review->getElementsByTagName ('resource_id', 0)->item(0)
                                  ->getChildNodes->toString;
            $self->{'dbh'}->do ("insert into $self->{'table'}review set ".
                                "baseURL=\'$baseURL\', ".
                                "id=\'$id\', ".
                                "metadataPrefix=\'$metadataPrefix\', ".
                                "internal_id=\'$identifier\', ".
                                "editor=\'$actor\', ".
                                "parent=\'$section\'"
                               );
            return $section;
         }

      }
   }

   return '';
}

sub getEditingSummary
{
   my ($self, $userid, $start, $stop) = @_;
   
   my @records = ();

   # define extents for summary
   if (! defined $start) { $start = 1; }
   if (! defined $stop) { $stop = 10; }
   $start--;
   $stop -= $start;

   my $sth = $self->{'dbh'}->prepare ("select sr.title,sr.internal_id,sr.status,sr.type,sr.level,e.role,e.deadline,sr.parent from $self->{'table'}sr as sr,$self->{'table'}editor as e where e.parent=sr.internal_id and e.id=\'$userid\' and e.role!=\'Reviewer\' order by sr.thread desc limit $start,$stop");
   $sth->execute;
   while ( my $rowref = $sth->fetchrow_hashref )
   {
      my $path = '';
      my $parent = $rowref->{'parent'};
      while ($parent ne '')
      {
         my $sth2 = $self->{'dbh'}->prepare ("select title,parent from $self->{'table'}sr where internal_id=\'$parent\'");
         $sth2->execute;
         my $rowref2 = $sth2->fetchrow_hashref;
         if ($rowref2)
         {
            if (defined $rowref2->{'title'})
            {
               if ($path ne '')
               {
                  $path = $rowref2->{'title'}.' / '.$path;
               }
               else
               {
                  $path = $rowref2->{'title'};
               }
            }
            if (defined $rowref2->{'parent'})
            {
               $parent = $rowref2->{'parent'};
            }
         }
         $sth2->finish;
      }
      
      my $srtype = 'section';
      if ($rowref->{'type'} eq 'r')
      {
         $srtype = 'resource';
      }
      push (@records, [ $rowref->{'internal_id'}, "<$srtype><title>$rowref->{'title'}</title><internal_id>$rowref->{'internal_id'}</internal_id><editor><role>$rowref->{'role'}</role><deadline>$rowref->{'deadline'}</deadline></editor><path>$path</path></$srtype>" ]);
   }
   $sth->finish;

   # get total count for number of items
   $sth = $self->{'dbh'}->prepare ("select count(*) from $self->{'table'}sr as sr,$self->{'table'}editor as e where e.parent=sr.internal_id and e.id=\'$userid\' and e.role!=\'Reviewer\'");
   $sth->execute;
   while (my @row = $sth->fetchrow_array)
   {
      push (@records, $row[0]);
   }
   $sth->finish;
   
   @records;
}

sub getReviewingSummary
{
   my ($self, $userid, $start, $stop) = @_;
   
   my @records = ();

   # define extents for summary
   if (! defined $start) { $start = 1; }
   if (! defined $stop) { $stop = 10; }
   $start--;
   $stop -= $start;

   my $sth = $self->{'dbh'}->prepare ("select sr.title,sr.internal_id,sr.status,sr.type,sr.level,e.role,e.deadline,sr.parent from $self->{'table'}sr as sr,$self->{'table'}editor as e where e.parent=sr.internal_id and e.id=\'$userid\' and e.role=\'Reviewer\' order by sr.thread desc limit $start,$stop");
   $sth->execute;
   while ( my $rowref = $sth->fetchrow_hashref )
   {
      my $path = '';
      my $parent = $rowref->{'parent'};
      while ($parent ne '')
      {
         my $sth2 = $self->{'dbh'}->prepare ("select title,parent from $self->{'table'}sr where internal_id=\'$parent\'");
         $sth2->execute;
         my $rowref2 = $sth2->fetchrow_hashref;
         if ($rowref2)
         {
            if (defined $rowref2->{'title'})
            {
               if ($path ne '')
               {
                  $path = $rowref2->{'title'}.' / '.$path;
               }
               else
               {
                  $path = $rowref2->{'title'};
               }
            }
            if (defined $rowref2->{'parent'})
            {
               $parent = $rowref2->{'parent'};
            }
         }
         $sth2->finish;
      }
      
      push (@records, [ $rowref->{'internal_id'}, "<resource><title>$rowref->{'title'}</title><internal_id>$rowref->{'internal_id'}</internal_id><editor><role>$rowref->{'role'}</role><id>$userid</id><deadline>$rowref->{'deadline'}</deadline></editor><path>$path</path></resource>" ]);
   }
   $sth->finish;
   
   # get total count for number of items
   $sth = $self->{'dbh'}->prepare ("select count(*) from $self->{'table'}sr as sr,$self->{'table'}editor as e where e.parent=sr.internal_id and e.id=\'$userid\' and e.role=\'Reviewer\'");
   $sth->execute;
   while (my @row = $sth->fetchrow_array)
   {
      push (@records, $row[0]);
   }
   $sth->finish;

   @records;
}

sub getResourceSummary
{
   my ($self, $resourceid) = @_;
   
   my $md = '';
   
   my $sth = $self->{'dbh'}->prepare ("select * from $self->{'table'}sr where internal_id=\'$resourceid\'");
   $sth->execute;
   my $rowref = $sth->fetchrow_hashref;
   $sth->finish;
   
   if ($rowref)
   {
      if ($rowref->{'type'} eq 'r')
      {
         $md .= '<resource>';
      }
      else
      {
         $md .= '<section>';
      }

      if (defined $rowref->{'title'}) { $md .= "<title>$rowref->{'title'}</title>"; }
      if (defined $rowref->{'internal_id'}) { $md .= "<internal_id>$rowref->{'internal_id'}</internal_id>"; }
      if (defined $rowref->{'status'}) { $md .= "<status>$rowref->{'status'}</status>"; }
      if (defined $rowref->{'submitter'}) { $md .= "<submitter>$rowref->{'submitter'}</submitter>"; }
      if (defined $rowref->{'baseURL'}) { $md .= "<baseURL>$rowref->{'baseURL'}</baseURL>"; }
      if (defined $rowref->{'id'}) { $md .= "<id>$rowref->{'id'}</id>"; }
      if (defined $rowref->{'metadataPrefix'}) { $md .= "<metadataPrefix>$rowref->{'metadataPrefix'}</metadataPrefix>"; }
      
      # list all editors
      my $sth3 = $self->{'dbh'}->prepare ("select * from $self->{'table'}editor where parent=\'$rowref->{'internal_id'}\'");
      $sth3->execute;
      while ( my $rowref3 = $sth3->fetchrow_hashref )
      {
         $md .= '<editor>';
         if (defined $rowref3->{'role'}) { $md .= "<role>$rowref3->{'role'}</role>"; }
         if (defined $rowref3->{'name'}) { $md .= "<name>$rowref3->{'name'}</name>"; }
         if (defined $rowref3->{'id'}) { $md .= "<id>$rowref3->{'id'}</id>"; }
         if (defined $rowref3->{'internal_id'}) { $md .= "<internal_id>$rowref3->{'internal_id'}</internal_id>"; }
         if (defined $rowref3->{'deadline'}) { $md .= "<deadline>$rowref3->{'deadline'}</deadline>"; }
         $md .= '</editor>';
      }
      $sth3->finish;

      # list all drafts
      $sth3 = $self->{'dbh'}->prepare ("select * from $self->{'table'}draft where parent=\'$rowref->{'internal_id'}\'");
      $sth3->execute;
      while ( my $rowref3 = $sth3->fetchrow_hashref )
      {
         $md .= '<draft>';
         if (defined $rowref3->{'internal_id'}) { $md .= "<internal_id>$rowref3->{'internal_id'}</internal_id>"; }
         if (defined $rowref3->{'title'}) { $md .= "<title>$rowref3->{'title'}</title>"; }
         if (defined $rowref3->{'baseURL'}) { $md .= "<baseURL>$rowref3->{'baseURL'}</baseURL>"; }
         if (defined $rowref3->{'id'}) { $md .= "<id>$rowref3->{'id'}</id>"; }
         if (defined $rowref3->{'metadataPrefix'}) { $md .= "<metadataPrefix>$rowref3->{'metadataPrefix'}</metadataPrefix>"; }
         $md .= '</draft>';
      }
      $sth3->finish;
      
      # list all reviews
      $sth3 = $self->{'dbh'}->prepare ("select * from $self->{'table'}review where parent=\'$rowref->{'internal_id'}\'");
      $sth3->execute;
      while ( my $rowref3 = $sth3->fetchrow_hashref )
      {
         $md .= '<review>';
         if (defined $rowref3->{'internal_id'}) { $md .= "<internal_id>$rowref3->{'internal_id'}</internal_id>"; }
         if (defined $rowref3->{'baseURL'}) { $md .= "<baseURL>$rowref3->{'baseURL'}</baseURL>"; }
         if (defined $rowref3->{'id'}) { $md .= "<id>$rowref3->{'id'}</id>"; }
         if (defined $rowref3->{'metadataPrefix'}) { $md .= "<metadataPrefix>$rowref3->{'metadataPrefix'}</metadataPrefix>"; }
         if (defined $rowref3->{'editor'}) { $md .= "<editor>$rowref3->{'editor'}</editor>"; }
         $md .= '</review>';
      }
      $sth3->finish;
      
      # list all children
      $sth3 = $self->{'dbh'}->prepare ("select title,internal_id,status,type,level,parent from $self->{'table'}sr where parent=\'$resourceid\' order by thread desc");
      $sth3->execute;
      while ( my $rowref3 = $sth3->fetchrow_hashref )
      {
         if ($rowref3->{'type'} eq 'r')
         {
            $md .= '<resource>';
         }
         else
         {
            $md .= '<section>';
         }
         if (defined $rowref3->{'title'}) { $md .= "<title>$rowref3->{'title'}</title>"; }
         if (defined $rowref3->{'internal_id'}) { $md .= "<internal_id>$rowref3->{'internal_id'}</internal_id>"; }
         if ($rowref3->{'type'} eq 'r')
         {
            $md .= '</resource>';
         }
         else
         {
            $md .= '</section>';
         }
      }
      $sth3->finish;
      
      if ($rowref->{'type'} eq 'r')
      {
         $md .= '</resource>';
      }
      else
      {
         $md .= '</section>';
      }

      my $parent = $rowref->{'parent'};

      while ($parent ne '') 
      {
         my $sth2 = $self->{'dbh'}->prepare ("select * from $self->{'table'}sr where internal_id=\'$parent\'");
         $sth2->execute;
         my $rowref2 = $sth2->fetchrow_hashref;
         $sth2->finish;
         
         my $md2 = '';
         
         if ($rowref2)
         {
            if (defined $rowref2->{'title'}) { $md2 .= "<title>$rowref2->{'title'}</title>"; }
            if (defined $rowref2->{'internal_id'}) { $md2 .= "<internal_id>$rowref2->{'internal_id'}</internal_id>"; }
            if (defined $rowref2->{'status'}) { $md2 .= "<status>$rowref2->{'status'}</status>"; }
            
            my $sth3 = $self->{'dbh'}->prepare ("select * from $self->{'table'}editor where parent=\'$rowref2->{'internal_id'}\'");
            $sth3->execute;
            while ( my $rowref3 = $sth3->fetchrow_hashref )
            {
               $md2 .= '<editor>';
               if (defined $rowref3->{'role'}) { $md2 .= "<role>$rowref3->{'role'}</role>"; }
               if (defined $rowref3->{'name'}) { $md2 .= "<name>$rowref3->{'name'}</name>"; }
               if (defined $rowref3->{'id'}) { $md2 .= "<id>$rowref3->{'id'}</id>"; }
               if (defined $rowref3->{'internal_id'}) { $md2 .= "<internal_id>$rowref3->{'internal_id'}</internal_id>"; }
               if (defined $rowref3->{'deadline'}) { $md2 .= "<deadline>$rowref3->{'deadline'}</deadline>"; }
               $md2 .= '</editor>';
            }
            $sth3->finish;
         }
         
         $md = '<section>'.$md2.$md.'</section>';
         
         $parent = $rowref2->{'parent'};
      }
   }
   
   $md;
}

sub getPath
{
   my ($self, $parent) = @_;
   
   my $path = '';
   while ($parent ne '')
   {
      my $sth = $self->{'dbh'}->prepare ("select title,parent from $self->{'table'}sr where internal_id=\'$parent\'");
      $sth->execute;
      my $rowref = $sth->fetchrow_hashref;
      if ($rowref)
      {
         if (defined $rowref->{'title'})
         {
            if ($path ne '')
            {
               $path = $rowref->{'title'}.' / '.$path;
            }
            else
            {
               $path = $rowref->{'title'};
            }
         }
         if (defined $rowref->{'parent'})
         {
            $parent = $rowref->{'parent'};
         }
      }
      $sth->finish;
   }
   
   $path;
}

sub getChildren
{
   my ($self, $resourceid, $start, $stop) = @_;

   my @records = ();
      
   # define extents for children list
   if (! defined $start) { $start = 1; }
   if (! defined $stop) { $stop = 10; }
   $start--;
   $stop -= $start;
   
   my $sth = $self->{'dbh'}->prepare ("select title,internal_id,status,type,level,parent from $self->{'table'}sr where parent=\'$resourceid\' order by thread desc limit $start,$stop");
   $sth->execute;
   while ( my $rowref = $sth->fetchrow_hashref )
   {
      my $path = $self->getPath ($rowref->{'parent'});
      push (@records, [ $rowref->{'internal_id'}, " <section><title>$rowref->{'title'}</title><path>$path</path><internal_id>$rowref->{'internal_id'}</internal_id></section> " ] );
   }
   $sth->finish;
   
   @records;
}

sub getOpen
{
   my ($self, $start, $stop) = @_;
   
   my @records = ();
   
   # define extents for list
   if (! defined $start) { $start = 1; }
   if (! defined $stop) { $stop = 10; }
   $start--;
   $stop -= $start;

   my $sth = $self->{'dbh'}->prepare ("select * from $self->{'table'}sr where status=\'open\' and type=\'s\' order by thread desc limit $start,$stop");
   $sth->execute;
   while ( my $rowref = $sth->fetchrow_hashref )
   {
      my $path = $self->getPath ($rowref->{'parent'});
      push (@records, [ $rowref->{'internal_id'}, '<section>'.
             '<title>'.$rowref->{'title'}.'</title>'.
             '<internal_id>'.$rowref->{'internal_id'}.'</internal_id>'.
             '<path>'.$path.'</path>'.
             '</section>' ] );
   }
   $sth->finish;
   
   # get total count for number of items
   $sth = $self->{'dbh'}->prepare ("select count(*) from $self->{'table'}sr where status=\'open\' and type=\'s\'");
   $sth->execute;
   while (my @row = $sth->fetchrow_array)
   {
      push (@records, $row[0]);
   }
   $sth->finish;

   @records;
}

sub getNotClosed
{
   my ($self, $start, $stop) = @_;

   my @records = ();
   
   # define extents for list
   if (! defined $start) { $start = 1; }
   if (! defined $stop) { $stop = 10; }
   $start--;
   $stop -= $start;

   my $sth = $self->{'dbh'}->prepare ("select * from $self->{'table'}sr where status!=\'closed\' and type=\'s\' order by thread desc limit $start,$stop");
   $sth->execute;
   while ( my $rowref = $sth->fetchrow_hashref )
   {
      my $path = $self->getPath ($rowref->{'parent'});
      push (@records, [ $rowref->{'internal_id'}, '<section>'.
             '<title>'.$rowref->{'title'}.'</title>'.
             '<internal_id>'.$rowref->{'internal_id'}.'</internal_id>'.
             '<path>'.$path.'</path>'.
             '</section>' ] );
   }
   $sth->finish;
   
   # get total count for number of items
   $sth = $self->{'dbh'}->prepare ("select count(*) from $self->{'table'}sr where status!=\'closed\' and type=\'s\'");
   $sth->execute;
   while (my @row = $sth->fetchrow_array)
   {
      push (@records, $row[0]);
   }
   $sth->finish;

   @records;
}

sub getSubmitted
{
   my ($self, $userid, $start, $stop) = @_;
   
   my @records = ();
   
   # define extents for list
   if (! defined $start) { $start = 1; }
   if (! defined $stop) { $stop = 10; }
   $start--;
   $stop -= $start;

   my $sth = $self->{'dbh'}->prepare ("select * from $self->{'table'}sr where submitter=\'$userid\' and type=\'r\' order by thread desc limit $start,$stop");
   $sth->execute;
   while ( my $rowref = $sth->fetchrow_hashref )
   {
      my $path = $self->getPath ($rowref->{'parent'});
      push (@records, [ $rowref->{'internal_id'}, '<resource>'.
             '<title>'.$rowref->{'title'}.'</title>'.
             '<internal_id>'.$rowref->{'internal_id'}.'</internal_id>'.
             '<path>'.$path.'</path>'.
             '<status>'.$rowref->{'status'}.'</status>'.
             '</resource>' ] );
   }
   $sth->finish;
   
   # get total count for number of items
   $sth = $self->{'dbh'}->prepare ("select count(*) from $self->{'table'}sr where submitter=\'$userid\' and type=\'r\'");
   $sth->execute;
   while (my @row = $sth->fetchrow_array)
   {
      push (@records, $row[0]);
   }
   $sth->finish;

   @records;
}


1;
